==> msp430-libc-1.0.20040716.tbz: msp430/share/mspgcc/examples/stackcheck/main.c <==
/**
 * See README.txt for details.
 * 
 * chris <cliechti@gmx.net>
 */
#include "hardware.h"

/** 
 * Address after the last used byte of the last segment in RAM (defined by
 * the linker)
 */
extern void *__noinit_end;

/** 
 * Access to the stack pointer.
 */
register void * __stackptr asm("r1");

/**
 * A constant that is unlikely to be a valid address/a value that occours
 * in the code
 */
#define STACK_GUARD_INIT        0xcf55

/** 
 * Place this variable after the end of the .no_init section, the last section
 * in the RAM. If the stack would grow too large, it wouldoverwrite this
 * variable.
 */
extern short stackGuard asm("__noinit_end"); 

/** 
 * Initilize the stack checking mechanisms.
 *  - fill unused memory with a pattern for manual max stack depth evaluation
 *  - set up the stackGuard
 */
inline void stackProtectionInit(void)
{
    unsigned short *address = &stackGuard;
    //fill free memory with a pattern for debug purpose (see max stack depth)
    for (; address < (unsigned short *)__stackptr; address++)
    {
        *address = 0x5aa5;
    }
    //init stack guard
    stackGuard = STACK_GUARD_INIT;
}

/**
 * This checks for stack overflows. It will perform a device reset if there
 * was an overflow.
 * 
 * It cannot help if the stack overflow is huge and overwrites vital
 * variables or exeeds the amount of RAM. But it works very well if important
 * stuff is at the beginning in the RAM and less important data at the end.
 * That way a log message could be printed, maybe even some error recovery
 * done. However, the simplest and probably best thing to do, is perform
 * a device reset.
 */
void stackCheck(void)
{
    if (stackGuard != STACK_GUARD_INIT)
    {
        //fatal error: the stack guard was altered -> stack overflow
        //~ log(LEVEL_FATAL, "Stack Overflow");
        //reset CPU
        WDTCTL = 0;                     //generate reset, trough wrong password
    }
}


/** 
 * A recursive function, used to fill the stack.
 */
void recurse(int howmany) {
	if (howmany) {
		recurse(--howmany);
	}
}

/** Globaly visible counter*/
int recusions = 0;						//recusion counter (input)

/**
 * The idea is to run this example in the debugger, look at the RAM contents
 * right after starting it (RAM should be all 0). Then single step down to
 * stackProtectionInit(), step over that function call. The RAM is now filled
 * with 0x5aa5. Set a breakpoint on the line "WDTCTL = 0;" in stackCheck() and
 * run the program (with "continue") until it hits that breakpoint. The RAM is
 * now filled up with return addresses of the last run.
 * 
 * The example just runs until it hits the stackGuard. With small modifications
 * or with manipulations in the debuger it can also show how the values in the
 * free memory are overwritten, which can be used to check how large the stack
 * grew at maximum.
 */
int main(void) {
    WDTCTL = WDTCTL_INIT;               //Init watchdog timer

    stackProtectionInit();              //set up stack protection now
    while (1) {                         //main loop, never ends...
    	recurse(recusions);				//do a recursive call that fills the stack
        stackCheck();                   //check for stack overflow
      	recusions++;					//add one recusion in the next run
    }
}

==> msp430-libc-1.0.20050912_1.tbz: msp430/share/mspgcc/examples/stackcheck/main.c <==
/**
 * See README.txt for details.
 * 
 * chris <cliechti@gmx.net>
 */
#include "hardware.h"

/**
 * A constant that is unlikely to be a valid address/a value that occours
 * in the code
 */
#define STACK_GUARD_INIT        0x0c55

extern unsigned short _end;  ///<Last used byte of the last segment in RAM (defined by the linker)

/** 
 * Initilize the stack checking mechanisms.
 *  - fill unused memory with a pattern for manual max stack depth evaluation
 *  - set up the stackGuard
 */
inline void stackProtectionInit(void)
{
    unsigned short *address = &_end;
    register void * __stackptr asm("r1");   ///<Access to the stack pointer
    //init stack guard
    (*address++) = STACK_GUARD_INIT;
    //fill free memory with a pattern for debug purpose (see max stack depth)
    while (address < (unsigned short *)__stackptr)
    {
        *address++ = 0x7a7a;
    }
}

/**
 * This checks for stack overflows. It will perform a device reset if there
 * was an overflow.
 * 
 * It cannot help if the stack overflow is huge and overwrites vital
 * variables or exeeds the amount of RAM. But it works very well if important
 * stuff is at the beginning in the RAM and less important data at the end.
 * That way a log message could be printed, maybe even some error recovery
 * done. However, the simplest and probably best thing to do, is perform
 * a device reset.
 */
void stackCheck(void)
{
    unsigned short *address = &_end;
    if ((*address) != STACK_GUARD_INIT)
    {
        //fatal error: the stack guard was altered -> stack overflow
        //~ log(LEVEL_FATAL, "Stack Overflow");
        //reset CPU
        WDTCTL = 0;                     //generate reset, trough wrong password
    }
}


/** 
 * A recursive function, used to fill the stack.
 */
void recurse(int howmany) {
    if (howmany) {
        recurse(--howmany);
    }
}

/** Globaly visible counter*/
int recusions = 0;                      //recusion counter (input)

/**
 * The idea is to run this example in the debugger, look at the RAM contents
 * right after starting it (RAM should be all 0). Then single step down to
 * stackProtectionInit(), step over that function call. The RAM is now filled
 * with 0x5aa5. Set a breakpoint on the line "WDTCTL = 0;" in stackCheck() and
 * run the program (with "continue") until it hits that breakpoint. The RAM is
 * now filled up with return addresses of the last run.
 * 
 * The example just runs until it hits the stackGuard. With small modifications
 * or with manipulations in the debuger it can also show how the values in the
 * free memory are overwritten, which can be used to check how large the stack
 * grew at maximum.
 */
int main(void) {
    WDTCTL = WDTCTL_INIT;               //Init watchdog timer

    stackProtectionInit();              //set up stack protection now
    while (1) {                         //main loop, never ends...
        recurse(recusions);             //do a recursive call that fills the stack
        stackCheck();                   //check for stack overflow
        recusions++;                    //add one recusion in the next run
    }
}
